﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using BEPUphysics;
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Boxes slide and spheres bounce.
    /// </summary>
    public class CoefficientsDemo : StandardDemo
    {
        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public CoefficientsDemo(DemosGame game)
            : base(game)
        {
            //The combine method changes how the properties of each entity in a collision are factored together.
            //Change them around a bit to alter the simulation.
            Space.SimulationSettings.CollisionDetection.FrictionBlendMethod = PropertyBlendMethod.Min; //Defaults to average
            Space.SimulationSettings.CollisionDetection.BouncinessBlendMethod = PropertyBlendMethod.Max;
            //Create the ground
            Entity toAdd = new Box(new Vector3(0, -.5f, 0), 20f, 1f, 20f);
            Space.Add(toAdd);
            //Bouncy balls 
            toAdd = new Sphere(new Vector3(-8, 10, 0), 1, 1);
            toAdd.Bounciness = .95f;
            Space.Add(toAdd);
            toAdd = new Sphere(new Vector3(-5, 10, 0), 1, 1);
            toAdd.Bounciness = .5f;
            Space.Add(toAdd);
            toAdd = new Sphere(new Vector3(-2, 10, 0), 1, 1);
            toAdd.Bounciness = .25f;
            Space.Add(toAdd);
            //Slide-y boxes
            toAdd = new Box(new Vector3(9, 1, 9), 1, 1, 1, 1);
            toAdd.DynamicFriction = 0;
            toAdd.LinearVelocity = new Vector3(-5, 0, 0);
            Space.Add(toAdd);
            toAdd = new Box(new Vector3(9, 1, 5), 1, 1, 1, 1);
            toAdd.DynamicFriction = .05f;
            toAdd.LinearVelocity = new Vector3(-5, 0, 0);
            Space.Add(toAdd);
            toAdd = new Box(new Vector3(9, 1, -5), 1, 1, 1, 1);
            toAdd.DynamicFriction = .2f;
            toAdd.LinearVelocity = new Vector3(-5, 0, 0);
            Space.Add(toAdd);
            toAdd = new Box(new Vector3(9, 1, -9), 1, 1, 1, 1);
            toAdd.DynamicFriction = .5f;
            toAdd.LinearVelocity = new Vector3(-5, 0, 0);
            Space.Add(toAdd);
            game.Camera.Position = new Vector3(0, 2, 30);
        }

        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string Name
        {
            get { return "Bounciness and Friction"; }
        }
    }
}