﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using BEPUphysics;
using BEPUphysics.DataStructures;
using BEPUphysics.Entities;
using BEPUphysicsDrawer.Models;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Box changes colors as it interacts with a detector volume.
    /// </summary>
    public class DetectorVolumeDemo : StandardDemo
    {
        //Box to change colors when it hits the detector volume.
        private readonly ModelDisplayObjectBase displayBox;

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public DetectorVolumeDemo(DemosGame game)
            : base(game)
        {
            var tubeModel = game.Content.Load<Model>("tube");
            TriangleMeshVertex[] modelVertices;
            int[] modelIndices;
            TriangleMesh.GetVerticesAndIndicesFromModel(tubeModel, out modelVertices, out modelIndices);
            var detectorVolume = new DetectorVolume(new TriangleMesh(modelVertices, modelIndices));
            Space.Add(detectorVolume);

            game.ModelDrawer.Add(detectorVolume.TriangleMesh);

            Entity toAdd = new Box(new Vector3(0, -10, 0), 1, 1, 1);
            toAdd.Tag = "noDisplayObject";
            displayBox = game.ModelDrawer.Add(toAdd);
            displayBox.TextureIndex = 0;
            game.ModelDrawer.IsWireframe = true;
            toAdd.LinearVelocity = new Vector3(0, 1, 0);
            Space.Add(toAdd); //Create the ground

            detectorVolume.EntityBeginsTouching += BeginsTouching;
            detectorVolume.EntityStopsTouching += StopsTouching;
            detectorVolume.VolumeBeginsContainingEntity += BeginsContaining;
            detectorVolume.VolumeStopsContainingEntity += StopsContaining;

            game.Camera.Position = new Vector3(0, 0, 22);
        }

        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string Name
        {
            get { return "Detector Volume"; }
        }

        private void StopsContaining(DetectorVolume volume, Entity entity)
        {
            displayBox.TextureIndex = 3;
        }

        private void BeginsContaining(DetectorVolume volume, Entity entity)
        {
            displayBox.TextureIndex = 2;
        }

        private void StopsTouching(Entity toucher, DetectorVolume volume)
        {
            displayBox.TextureIndex = 0;
        }

        private void BeginsTouching(Entity toucher, DetectorVolume volume)
        {
            displayBox.TextureIndex = 1;
        }

        public override void CleanUp()
        {
            Game.ModelDrawer.IsWireframe = false;
            base.CleanUp();
        }
    }
}