﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using BEPUphysics;
using BEPUphysics.DataStructures;
using BEPUphysics.Entities;
using BEPUphysicsDrawer.Models;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// Put the fish in a barrel, and a box moves.
    /// Game of the year!
    /// </summary>
    public class FishInABarrelDemo : StandardDemo
    {
        //Some entities to keep track of for the demo's events.
        private readonly Entity acceptedTrigger;
        private readonly Entity movedBox;

        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public FishInABarrelDemo(DemosGame game)
            : base(game)
        {
            game.Camera.Position = new Vector3(0, 7, 30);

            var detector = new Box(new Vector3(0, 0, 0), 1.5f, 1.5f, 1.5f);
            detector.CollisionRules.Personal = CollisionRule.NoResponse;
            acceptedTrigger = new Box(new Vector3(5, 0, 0), 1.6f, .7f, .4f, 1);

            detector.Tag = "noDisplayObject";
            acceptedTrigger.Tag = "noDisplayObject";
            Space.Add(detector);
            Space.Add(acceptedTrigger);

            var fish = game.Content.Load<Model>("fish");
            game.ModelDrawer.Add(new DisplayEntityModelTest(acceptedTrigger, fish, game.ModelDrawer));

            var barrelAndPlatform = game.Content.Load<Model>("barrelAndPlatform");
            StaticTriangleGroup.StaticTriangleGroupVertex[] staticTriangleVertices;
            int[] staticTriangleIndices;
            StaticTriangleGroup.GetVerticesAndIndicesFromModel(barrelAndPlatform, out staticTriangleVertices, out staticTriangleIndices);
            //Note that the final 'margin' parameter is optional, but can be used to specify a collision margin on triangles in the static triangle group.
            var mesh = new TriangleMesh(staticTriangleVertices, staticTriangleIndices, 0);
            var fishDepositoryGroup = new StaticTriangleGroup(mesh);
            fishDepositoryGroup.CollisionRules.SpecificEntities.Add(detector, CollisionRule.NoPair);
            Space.Add(fishDepositoryGroup);
            game.ModelDrawer.Add(mesh);


            movedBox = new Box(new Vector3(-4, 5, 0), 1, 1, 1, 1);
            detector.Space.Add(movedBox);
            detector.EventManager.InitialCollisionDetected += InitialCollisionDetected;
            detector.EventManager.CollisionEnded += CollisionEnded;
        }

        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string Name
        {
            get { return "Fish in a Barrel"; }
        }


        public void InitialCollisionDetected(Entity sender, Entity other, CollisionPair collisionPair)
        {
            if (other == acceptedTrigger)
            {
                //If the detector collided with the accepted trigger, move the box.
                movedBox.CenterPosition = new Vector3(4, 5, 0);
                movedBox.OrientationQuaternion = Quaternion.Identity;
                movedBox.LinearVelocity = Vector3.Zero;
                movedBox.AngularVelocity = Vector3.Zero;
            }
        }

        public void CollisionEnded(Entity sender, Entity other, CollisionPair collisionPair)
        {
            if (other == acceptedTrigger)
            {
                //If the detector ceases to collide, get rid of the spawned box.
                movedBox.CenterPosition = new Vector3(-4, 5, 0);
                movedBox.OrientationQuaternion = Quaternion.Identity;
                movedBox.LinearVelocity = Vector3.Zero;
                movedBox.AngularVelocity = Vector3.Zero;
            }
        }

        public override void DrawUI()
        {
            Game.DataTextDrawer.Draw("Put the fish in the barrel!", new Vector2(500, 50));
        }
    }
}