﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using BEPUphysics.Entities;
using Microsoft.Xna.Framework;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// A set of jenga blocks.
    /// </summary>
    public class JengaDemo : StandardDemo
    {
        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public JengaDemo(DemosGame game)
            : base(game)
        {
            Space.Remove(kapow);
            //Have to shrink the ball a little to make it fit between jenga tower blocks.
            kapow = new Sphere(new Vector3(-11000, 0, 0), .4f, 20);
            kapow.CollisionMargin = .2f;
            Space.Add(kapow);
            int numBlocksTall = 18; //How many 'stories' tall.
            float blockWidth = 4; //Total width/length of the tower.
            float blockHeight = 4 / 3f;
            Entity toAdd;

            //The default number of iterations is 10, but this particular demo likes more for extra stability.
            Space.SimulationSettings.CollisionResponse.Iterations = 15;

            for (int i = 0; i < numBlocksTall; i++)
            {
                if (i % 2 == 0)
                {
                    for (int j = 0; j < 3; j++)
                    {
                        toAdd = new Box(new Vector3(
                                            j * (blockWidth / 3 + Space.SimulationSettings.CollisionDetection.DefaultMargin * 2) - blockWidth / 3f -
                                            Space.SimulationSettings.CollisionDetection.DefaultMargin * 2,
                                            blockHeight / 2 + Space.SimulationSettings.CollisionDetection.DefaultMargin * 2 +
                                            i * (blockHeight + Space.SimulationSettings.CollisionDetection.DefaultMargin * 2),
                                            0),
                                        blockWidth / 3, blockHeight, blockWidth, 10);
                        Space.Add(toAdd);
                    }
                }
                else
                {
                    for (int j = 0; j < 3; j++)
                    {
                        toAdd = new Box(new Vector3(
                                            0,
                                            blockHeight / 2 + Space.SimulationSettings.CollisionDetection.DefaultMargin * 2 +
                                            (i) * (blockHeight + Space.SimulationSettings.CollisionDetection.DefaultMargin * 2),
                                            j * (blockWidth / 3 + Space.SimulationSettings.CollisionDetection.DefaultMargin * 2) - blockWidth / 3f -
                                            Space.SimulationSettings.CollisionDetection.DefaultMargin * 2),
                                        blockWidth, blockHeight, blockWidth / 3, 10);
                        Space.Add(toAdd);
                    }
                }
            }
            Space.Add(new Box(new Vector3(0, -.5f, 0), 70, 1, 70));
            game.Camera.Position = new Vector3(0, 8, 32);
        }

        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string Name
        {
            get { return "Jenga"; }
        }
    }
}