﻿/*
      Copyright (C) 2010 Bepu Entertainment LLC.

      This software source code is provided 'as-is', without 
      any express or implied warranty.  In no event will the authors be held 
      liable for any damages arising from the use of this software.

      Permission is granted to anyone to use this software for any purpose,
      including commercial applications, and to alter it and redistribute it
      freely, subject to the following restrictions:

      1. The origin of this software must not be misrepresented; you must not
         claim that you wrote the original software. If you use this software
         in a product, an acknowledgment in the product documentation would be
         appreciated but is not required.
      2. Altered source versions must be plainly marked as such, and must not be
         misrepresented as being the original software.
      3. This notice may not be removed or altered from any source distribution.

    Contact us at:
    contact@bepu-games.com
 */
using System;
using BEPUphysics.Entities;
using BEPUphysicsDemos.SampleCode;
using Microsoft.Xna.Framework;

namespace BEPUphysicsDemos.Demos
{
    /// <summary>
    /// A spaceship blasts off into the sky (void).
    /// </summary>
    public class SpaceshipDemo : StandardDemo
    {
        /// <summary>
        /// Constructs a new demo.
        /// </summary>
        /// <param name="game">Game owning this demo.</param>
        public SpaceshipDemo(DemosGame game)
            : base(game)
        {
            //Build the ship
            var shipFuselage = new Cylinder(new Vector3(0, 5, 0), 3, .7f, 4);
            var shipNose = new Cone(new Vector3(0, 7.00f, 0), 2, .7f, 2);
            var shipWing = new Box(new Vector3(0, 5, 0), 5f, 2, .2f, 3);
            var shipThrusters = new Cone(new Vector3(0, 3.25f, 0), 1, .5f, 1);


            var ship = new CompoundBody();
            ship.AddBody(shipFuselage);
            ship.AddBody(shipNose);
            ship.AddBody(shipWing);
            ship.AddBody(shipThrusters);

            //Setup the launch pad and ramp
            Entity toAdd = new Box(new Vector3(10, 4, 0), 26, 1f, 6);
            Space.Add(toAdd);
            toAdd = new Box(new Vector3(32, 7.8f, 0), 20, 1, 6);
            toAdd.OrientationQuaternion = Quaternion.CreateFromAxisAngle(Vector3.Forward, -(float) Math.PI / 8);
            Space.Add(toAdd);
            toAdd = new Box(new Vector3(32, 8.8f, -3.5f), 20, 1, 1);
            toAdd.OrientationQuaternion = Quaternion.CreateFromAxisAngle(Vector3.Forward, -(float) Math.PI / 8);
            Space.Add(toAdd);
            toAdd = new Box(new Vector3(32, 8.8f, 3.5f), 20, 1, 1);
            toAdd.OrientationQuaternion = Quaternion.CreateFromAxisAngle(Vector3.Forward, -(float) Math.PI / 8);
            Space.Add(toAdd);
            toAdd = new Box(new Vector3(-2.75f, 5.5f, 0), .5f, 2f, 3);
            Space.Add(toAdd);

            //Blast-off!
            ship.AngularDamping = .95f; //Helps keep the rocket on track for a little while longer :D
            var thruster = new Thruster(ship, new Vector3(0, -2, 0), new Vector3(0, 300, 0), 0);
            Space.Add(thruster);
            ship.OrientationQuaternion = Quaternion.CreateFromAxisAngle(Vector3.Right, (float) Math.PI / 2) * Quaternion.CreateFromAxisAngle(Vector3.Forward, (float) Math.PI / 2);
            Space.Add(ship);


            game.Camera.Position = new Vector3(-14, 12, 25);
            game.Camera.Yaw = (float) Math.PI / -4;
        }

        /// <summary>
        /// Gets the name of the simulation.
        /// </summary>
        public override string Name
        {
            get { return "Spaceship"; }
        }
    }
}